=== modified file 'NEWS'
--- NEWS	2006-07-03 13:48:52 +0000
+++ NEWS	2006-07-06 13:51:22 +0000
@@ -69,6 +69,9 @@
    * 'branches.conf' has been changed to 'locations.conf', since it can apply
      to more locations than just branch locations.
      (Aaron Bentley)
+   
+   * 'bzr whoami' can now be used to set your identity from the command line,
+     for a branch or globally.  (Robey Pointer)
 
    * 'bzr checkout' now aliased to 'bzr co', and 'bzr annotate' to 'bzr ann'.
      (Michael Ellerman)
@@ -99,6 +102,10 @@
     * Cleanup win32 test suite, and general cleanup of places where
       file handles were being held open. (John Arbash Meinel)
 
+    * When specifying filenames for 'diff -r x..y', the name of the file in the
+      working directory can be used, even if its name is different in both x
+      and y.
+
   INTERNALS:
 
     * Combine the ignore rules into a single regex rather than looping over

=== modified file 'bzrlib/benchmarks/__init__.py'
--- bzrlib/benchmarks/__init__.py	2006-06-23 15:27:25 +0000
+++ bzrlib/benchmarks/__init__.py	2006-07-06 02:27:26 +0000
@@ -17,11 +17,11 @@
 
 """Benchmark test suite for bzr."""
 
-import bzrlib
+from bzrlib import bzrdir, plugin
 from bzrlib.tests.TestUtil import TestLoader
-from bzrlib.bzrdir import BzrDir
 from bzrlib.tests.blackbox import ExternalBase
 
+
 class Benchmark(ExternalBase):
 
     def make_kernel_like_tree(self, url=None):
@@ -35,8 +35,8 @@
         # we simulate this by three levels of dirs named 0-7, givin 512 dirs,
         # and 20 files each.
         if url is not None:
-            b = bzrlib.bzrdir.BzrDir.create_branch_convenience(url)
-            d = bzrlib.bzrdir.BzrDir.create('.')
+            b = bzrdir.BzrDir.create_branch_convenience(url)
+            d = bzrdir.BzrDir.create('.')
             bzrlib.branch.BranchReferenceFormat().initialize(d, b)
             d.create_workingtree()
         else:
@@ -118,4 +118,11 @@
                    'bzrlib.benchmarks.bench_transform',
                    'bzrlib.benchmarks.bench_workingtree',
                    ]
-    return TestLoader().loadTestsFromModuleNames(testmod_names)
+    suite = TestLoader().loadTestsFromModuleNames(testmod_names) 
+
+    # Load any benchmarks from plugins
+    for name, module in plugin.all_plugins().items():
+        if getattr(module, 'bench_suite', None) is not None:
+            suite.addTest(module.bench_suite())
+
+    return suite

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-07-03 13:35:59 +0000
+++ bzrlib/builtins.py	2006-07-04 15:04:11 +0000
@@ -1788,19 +1788,41 @@
 
 
 class cmd_whoami(Command):
-    """Show bzr user id."""
-    takes_options = ['email']
+    """Show or set bzr user id.
+    
+    examples:
+        bzr whoami --email
+        bzr whoami 'Frank Chu <fchu@example.com>'
+    """
+    takes_options = [ Option('email',
+                             help='display email address only'),
+                      Option('branch',
+                             help='set identity for the current branch instead of '
+                                  'globally'),
+                    ]
+    takes_args = ['name?']
+    encoding_type = 'replace'
     
     @display_command
-    def run(self, email=False):
-        try:
-            c = WorkingTree.open_containing(u'.')[0].branch.get_config()
-        except NotBranchError:
+    def run(self, email=False, branch=False, name=None):
+        if name is None:
+            # use branch if we're inside one; otherwise global config
+            try:
+                c = Branch.open_containing('.')[0].get_config()
+            except NotBranchError:
+                c = config.GlobalConfig()
+            if email:
+                self.outf.write(c.user_email() + '\n')
+            else:
+                self.outf.write(c.username() + '\n')
+            return
+
+        # use global config unless --branch given
+        if branch:
+            c = Branch.open_containing('.')[0].get_config()
+        else:
             c = config.GlobalConfig()
-        if email:
-            print c.user_email()
-        else:
-            print c.username()
+        c.set_user_option('email', name)
 
 
 class cmd_nick(Command):

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2006-07-02 01:21:07 +0000
+++ bzrlib/config.py	2006-07-04 15:04:11 +0000
@@ -337,6 +337,19 @@
     def __init__(self):
         super(GlobalConfig, self).__init__(config_filename)
 
+    def set_user_option(self, option, value):
+        """Save option and its value in the configuration."""
+        # FIXME: RBC 20051029 This should refresh the parser and also take a
+        # file lock on bazaar.conf.
+        conf_dir = os.path.dirname(self._get_filename())
+        ensure_config_dir_exists(conf_dir)
+        if 'DEFAULT' not in self._get_parser():
+            self._get_parser()['DEFAULT'] = {}
+        self._get_parser()['DEFAULT'][option] = value
+        f = open(self._get_filename(), 'wb')
+        self._get_parser().write(f)
+        f.close()
+
 
 class LocationConfig(IniBasedConfig):
     """A configuration object that gives the policy for a location."""
@@ -611,13 +624,25 @@
         uid = os.getuid()
         w = pwd.getpwuid(uid)
 
-        try:
-            gecos = w.pw_gecos.decode(bzrlib.user_encoding)
-            username = w.pw_name.decode(bzrlib.user_encoding)
-        except UnicodeDecodeError:
-            # We're using pwd, therefore we're on Unix, so /etc/passwd is ok.
-            raise errors.BzrError("Can't decode username in " \
-                    "/etc/passwd as %s." % bzrlib.user_encoding)
+        # we try utf-8 first, because on many variants (like Linux),
+        # /etc/passwd "should" be in utf-8, and because it's unlikely to give
+        # false positives.  (many users will have their user encoding set to
+        # latin-1, which cannot raise UnicodeError.)
+        try:
+            gecos = w.pw_gecos.decode('utf-8')
+            encoding = 'utf-8'
+        except UnicodeError:
+            try:
+                gecos = w.pw_gecos.decode(bzrlib.user_encoding)
+                encoding = bzrlib.user_encoding
+            except UnicodeError:
+                raise errors.BzrCommandError('Unable to determine your name.  '
+                   'Use "bzr whoami" to set it.')
+        try:
+            username = w.pw_name.decode(encoding)
+        except UnicodeError:
+            raise errors.BzrCommandError('Unable to determine your name.  '
+                'Use "bzr whoami" to set it.')
 
         comma = gecos.find(',')
         if comma == -1:

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2006-06-16 17:03:42 +0000
+++ bzrlib/delta.py	2006-07-06 13:51:22 +0000
@@ -16,6 +16,7 @@
 
 from bzrlib.inventory import InventoryEntry
 from bzrlib.trace import mutter
+from bzrlib import tree
 
 
 class TreeDelta(object):
@@ -142,7 +143,9 @@
             show_list(self.unchanged)
 
 
-def compare_trees(old_tree, new_tree, want_unchanged=False, specific_files=None):
+def compare_trees(old_tree, new_tree, want_unchanged=False, 
+                  specific_files=None, extra_trees=None, 
+                  require_versioned=False):
     """Describe changes from one tree to another.
 
     Returns a TreeDelta with details of added, modified, renamed, and
@@ -157,9 +160,18 @@
         the next.
 
     specific_files
-        If true, only check for changes to specified names or
-        files within them.  Any unversioned files given have no effect
-        (but this might change in the future).
+        If supplied, only check for changes to specified names or
+        files within them.  When mapping filenames to ids, all matches in all
+        trees (including optional extra_trees) are used, and all children of
+        matched directories are included.
+
+    extra_trees
+        If non-None, a list of more trees to use for looking up file_ids from
+        paths
+
+    require_versioned
+        If true, an all files are required to be versioned, and
+        PathsNotVersionedError will be thrown if they are not.
     """
     # NB: show_status depends on being able to pass in non-versioned files and
     # report them as unknown
@@ -167,15 +179,20 @@
     try:
         new_tree.lock_read()
         try:
+            trees = (new_tree, old_tree)
+            if extra_trees is not None:
+                trees = trees + tuple(extra_trees)
+            specific_file_ids = tree.find_ids_across_trees(specific_files, 
+                trees, require_versioned=require_versioned)
             return _compare_trees(old_tree, new_tree, want_unchanged,
-                                  specific_files)
+                                  specific_file_ids)
         finally:
             new_tree.unlock()
     finally:
         old_tree.unlock()
 
 
-def _compare_trees(old_tree, new_tree, want_unchanged, specific_files):
+def _compare_trees(old_tree, new_tree, want_unchanged, specific_file_ids):
 
     from osutils import is_inside_any
     
@@ -186,8 +203,6 @@
 
     # TODO: Rather than iterating over the whole tree and then filtering, we
     # could diff just the specified files (if any) and their subtrees.  
-    # Perhaps should take a list of file-ids instead?   Need to indicate any
-    # ids or names which were not found in the trees.
 
     old_files = old_tree.list_files()
     new_files = new_tree.list_files()
@@ -214,9 +229,9 @@
         if old_entry.kind == 'root_directory':
             return
 
-        if specific_files:
-            if (not is_inside_any(specific_files, old_path)
-                and not is_inside_any(specific_files, new_path)):
+        if specific_file_ids:
+            if (old_file_id not in specific_file_ids and 
+                new_file_id not in specific_file_ids):
                 return
 
         # temporary hack until all entries are populated before clients 
@@ -314,13 +329,13 @@
 
     # Now we have a set of added and removed files, mark them all
     for old_path, old_entry in removed.itervalues():
-        if specific_files:
-            if not is_inside_any(specific_files, old_path):
+        if specific_file_ids:
+            if not old_entry.file_id in specific_file_ids:
                 continue
         delta.removed.append((old_path, old_entry.file_id, old_entry.kind))
     for new_path, new_entry in added.itervalues():
-        if specific_files:
-            if not is_inside_any(specific_files, new_path):
+        if specific_file_ids:
+            if not new_entry.file_id in specific_file_ids:
                 continue
         delta.added.append((new_path, new_entry.file_id, new_entry.kind))
 

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-06-20 05:32:16 +0000
+++ bzrlib/diff.py	2006-07-05 05:18:06 +0000
@@ -269,15 +269,21 @@
         new_tree = tree
     else:
         new_tree = spec_tree(new_revision_spec)
+    if new_tree is not tree:
+        extra_trees = (tree,)
+    else:
+        extra_trees = None
 
     return show_diff_trees(old_tree, new_tree, sys.stdout, specific_files,
                            external_diff_options,
-                           old_label=old_label, new_label=new_label)
+                           old_label=old_label, new_label=new_label,
+                           extra_trees=extra_trees)
 
 
 def show_diff_trees(old_tree, new_tree, to_file, specific_files=None,
                     external_diff_options=None,
-                    old_label='a/', new_label='b/'):
+                    old_label='a/', new_label='b/',
+                    extra_trees=None):
     """Show in text form the changes from one tree to another.
 
     to_files
@@ -285,6 +291,9 @@
 
     external_diff_options
         If set, use an external GNU diff and pass these options.
+
+    extra_trees
+        If set, more Trees to use for looking up file ids
     """
     old_tree.lock_read()
     try:
@@ -292,7 +301,8 @@
         try:
             return _show_diff_trees(old_tree, new_tree, to_file,
                                     specific_files, external_diff_options,
-                                    old_label=old_label, new_label=new_label)
+                                    old_label=old_label, new_label=new_label,
+                                    extra_trees=extra_trees)
         finally:
             new_tree.unlock()
     finally:
@@ -301,7 +311,7 @@
 
 def _show_diff_trees(old_tree, new_tree, to_file,
                      specific_files, external_diff_options, 
-                     old_label='a/', new_label='b/' ):
+                     old_label='a/', new_label='b/', extra_trees=None):
 
     # GNU Patch uses the epoch date to detect files that are being added
     # or removed in a diff.
@@ -310,8 +320,6 @@
     # TODO: Generation of pseudo-diffs for added/deleted files could
     # be usefully made into a much faster special case.
 
-    _raise_if_doubly_unversioned(specific_files, old_tree, new_tree)
-
     if external_diff_options:
         assert isinstance(external_diff_options, basestring)
         opts = external_diff_options.split()
@@ -321,7 +329,8 @@
         diff_file = internal_diff
     
     delta = compare_trees(old_tree, new_tree, want_unchanged=False,
-                          specific_files=specific_files)
+                          specific_files=specific_files, 
+                          extra_trees=extra_trees, require_versioned=True)
 
     has_changes = 0
     for path, file_id, kind in delta.removed:
@@ -379,17 +388,6 @@
     return time.strftime('%Y-%m-%d %H:%M:%S +0000', tm)
 
 
-def _raise_if_doubly_unversioned(specific_files, old_tree, new_tree):
-    """Complain if paths are not versioned in either tree."""
-    if not specific_files:
-        return
-    old_unversioned = old_tree.filter_unversioned_files(specific_files)
-    new_unversioned = new_tree.filter_unversioned_files(specific_files)
-    unversioned = old_unversioned.intersection(new_unversioned)
-    if unversioned:
-        raise errors.PathsNotVersionedError(sorted(unversioned))
-    
-
 def _raise_if_nonexistent(paths, old_tree, new_tree):
     """Complain if paths are not in either inventory or tree.
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-06-25 21:04:37 +0000
+++ bzrlib/repository.py	2006-07-04 17:23:00 +0000
@@ -466,8 +466,14 @@
     def get_revision_graph(self, revision_id=None):
         """Return a dictionary containing the revision graph.
         
+        :param revision_id: The revision_id to get a graph from. If None, then
+        the entire revision graph is returned. This is a deprecated mode of
+        operation and will be removed in the future.
         :return: a dictionary of revision_id->revision_parents_list.
         """
+        # special case NULL_REVISION
+        if revision_id == NULL_REVISION:
+            return {}
         weave = self.get_inventory_weave()
         all_revisions = self._eliminate_revisions_not_present(weave.versions())
         entire_graph = dict([(node, weave.get_parents(node)) for 
@@ -501,7 +507,10 @@
             required = set([])
         else:
             pending = set(revision_ids)
-            required = set(revision_ids)
+            # special case NULL_REVISION
+            if NULL_REVISION in pending:
+                pending.remove(NULL_REVISION)
+            required = set(pending)
         done = set([])
         while len(pending):
             revision_id = pending.pop()
@@ -873,9 +882,15 @@
     @needs_read_lock
     def get_revision_graph(self, revision_id=None):
         """Return a dictionary containing the revision graph.
-        
+
+        :param revision_id: The revision_id to get a graph from. If None, then
+        the entire revision graph is returned. This is a deprecated mode of
+        operation and will be removed in the future.
         :return: a dictionary of revision_id->revision_parents_list.
         """
+        # special case NULL_REVISION
+        if revision_id == NULL_REVISION:
+            return {}
         weave = self._get_revision_vf()
         entire_graph = weave.get_graph()
         if revision_id is None:
@@ -909,7 +924,10 @@
             required = set([])
         else:
             pending = set(revision_ids)
-            required = set(revision_ids)
+            # special case NULL_REVISION
+            if NULL_REVISION in pending:
+                pending.remove(NULL_REVISION)
+            required = set(pending)
         done = set([])
         while len(pending):
             revision_id = pending.pop()

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2006-06-20 03:57:11 +0000
+++ bzrlib/revision.py	2006-07-04 17:23:00 +0000
@@ -240,6 +240,8 @@
                     pb=DummyProgress()):
     if None in (revision_a, revision_b):
         return None
+    if NULL_REVISION in (revision_a, revision_b):
+        return NULL_REVISION
     # trivial optimisation
     if revision_a == revision_b:
         return revision_a

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-07-03 08:12:23 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-07-04 15:04:11 +0000
@@ -84,6 +84,7 @@
                      'bzrlib.tests.blackbox.test_update',
                      'bzrlib.tests.blackbox.test_upgrade',
                      'bzrlib.tests.blackbox.test_versioning',
+                     'bzrlib.tests.blackbox.test_whoami',
                      ]
     test_encodings = [
         'bzrlib.tests.blackbox.test_non_ascii',

=== modified file 'bzrlib/tests/blackbox/test_diff.py'
--- bzrlib/tests/blackbox/test_diff.py	2006-06-30 15:40:53 +0000
+++ bzrlib/tests/blackbox/test_diff.py	2006-07-05 05:03:34 +0000
@@ -23,6 +23,7 @@
 
 import bzrlib
 from bzrlib.branch import Branch
+from bzrlib import workingtree
 from bzrlib.tests.blackbox import ExternalBase
 
 
@@ -173,6 +174,15 @@
         output = self.run_bzr_captured(['diff', '-r', '1..', 'branch1'], retcode=1)
         self.assertTrue('\n-original line\n+new line\n' in output[0])
 
+    def test_diff_across_rename(self):
+        """The working tree path should always be considered for diffing"""
+        self.make_example_branch()
+        self.run_bzr('diff', '-r', '0..1', 'hello', retcode=1)
+        wt = workingtree.WorkingTree.open_containing('.')[0]
+        wt.rename_one('hello', 'hello1')
+        self.run_bzr('diff', 'hello1', retcode=1)
+        self.run_bzr('diff', '-r', '0..1', 'hello1', retcode=1)
+
 
 class TestCheckoutDiff(TestDiff):
 

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2006-06-27 21:05:08 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2006-07-03 17:22:41 +0000
@@ -22,12 +22,12 @@
 
 import bzrlib
 import bzrlib.osutils as osutils
-from bzrlib.tests import TestCaseInTempDir, TestSkipped
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
 from bzrlib.trace import mutter, note
 import bzrlib.urlutils as urlutils
 
 
-class TestNonAscii(TestCaseInTempDir):
+class TestNonAscii(TestCaseWithTransport):
     """Test that bzr handles files/committers/etc which are non-ascii."""
 
     def setUp(self):
@@ -72,18 +72,19 @@
                                    ' filesystem encoding "%s")')
                                    % (thing, terminal_enc, fs_enc))
 
-        bzr('init')
+        wt = self.make_branch_and_tree('.')
         open('a', 'wb').write('foo\n')
-        bzr('add', 'a')
-        bzr('commit', '-m', 'adding a')
+        wt.add('a')
+        wt.commit('adding a')
 
         open('b', 'wb').write('non-ascii \xFF\xFF\xFC\xFB\x00 in b\n')
-        bzr('add', 'b')
-        bzr('commit', '-m', self.info['message'])
+        wt.add('b')
+        wt.commit(self.info['message'])
 
         open(fname, 'wb').write('unicode filename\n')
-        bzr('add', fname)
-        bzr('commit', '-m', u'And a unicode file\n')
+        wt.add(fname)
+        wt.commit(u'And a unicode file\n')
+        self.wt = wt
 
     def test_status(self):
         bzr = self.run_bzr_decode
@@ -184,9 +185,12 @@
         self.failIfExists('a')
         self.failUnlessExists(fname2)
 
-        bzr('commit', '-m', 'renamed to non-ascii')
+        # After 'mv' we need to re-open the working tree
+        self.wt = self.wt.bzrdir.open_workingtree()
+        self.wt.commit('renamed to non-ascii')
 
-        bzr('mkdir', dirname)
+        os.mkdir(dirname)
+        self.wt.add(dirname)
         txt = bzr('mv', fname1, fname2, dirname)
         self.assertEqual([u'%s => %s/%s' % (fname1, dirname, fname1),
                           u'%s => %s/%s' % (fname2, dirname, fname2)]
@@ -211,12 +215,14 @@
 
         dirname1 = self.info['directory']
         dirname2 = self.info['directory'] + '2'
-        bzr('branch', '.', dirname1)
-        bzr('branch', dirname1, dirname2)
+        url1 = urlutils.local_path_to_url(dirname1)
+        url2 = urlutils.local_path_to_url(dirname2)
+        out_bzrdir = self.wt.bzrdir.sprout(url1)
+        out_bzrdir.sprout(url2)
 
         os.chdir(dirname1)
         open('a', 'ab').write('more text\n')
-        bzr('commit', '-m', 'mod a')
+        self.wt.commit('mod a')
 
         pwd = osutils.getcwd()
 
@@ -227,7 +233,7 @@
 
         os.chdir('../' + dirname1)
         open('a', 'ab').write('and yet more\n')
-        bzr('commit', '-m', 'modifying a by ' + self.info['committer'])
+        self.wt.commit(u'modifying a by ' + self.info['committer'])
 
         os.chdir('../' + dirname2)
         # We should be able to pull, even if our encoding is bad
@@ -246,18 +252,20 @@
         bzr('push', dirname)
 
         open('a', 'ab').write('adding more text\n')
-        bzr('commit', '-m', 'added some stuff')
+        self.wt.commit('added some stuff')
 
         # TODO: check the output text is properly encoded
         bzr('push')
 
         f = open('a', 'ab')
-        f.write('and a bit more: ')
-        f.write(dirname.encode('utf-8'))
-        f.write('\n')
-        f.close()
+        try:
+            f.write('and a bit more: ')
+            f.write(dirname.encode('utf-8'))
+            f.write('\n')
+        finally:
+            f.close()
 
-        bzr('commit', '-m', u'Added some ' + dirname)
+        self.wt.commit('Added some ' + dirname)
         bzr('push', '--verbose', encoding='ascii')
 
         bzr('push', '--verbose', dirname + '2')
@@ -306,11 +314,13 @@
         # Create a directory structure
         fname = self.info['filename']
         dirname = self.info['directory']
-        bzr('mkdir', 'base')
-        bzr('mkdir', 'base/' + dirname)
+        os.mkdir('base')
+        os.mkdir('base/' + dirname)
+        self.wt.add('base')
+        self.wt.add('base/'+dirname)
         path = '/'.join(['base', dirname, fname])
-        bzr('mv', fname, path)
-        bzr('commit', '-m', 'moving things around')
+        self.wt.rename_one(fname, path)
+        self.wt.commit('moving things around')
 
         txt = bzr('file-path', path)
 
@@ -345,7 +355,7 @@
 
         fname = self.info['filename']
         os.remove(fname)
-        bzr('rm', fname)
+        self.wt.remove(fname)
 
         txt = bzr('deleted')
         self.assertEqual(fname+'\n', txt)
@@ -374,7 +384,7 @@
 
         fname = self.info['filename'] + '2'
         open(fname, 'wb').write('added\n')
-        bzr('add', fname)
+        self.wt.add(fname)
 
         txt = bzr('added')
         self.assertEqual(fname+'\n', txt)
@@ -385,9 +395,10 @@
         bzr = self.run_bzr_decode
 
         dirname = self.info['directory']
+        url = urlutils.local_path_to_url(dirname)
         bzr('root')
 
-        bzr('branch', u'.', dirname)
+        self.wt.bzrdir.sprout(url)
 
         os.chdir(dirname)
 
@@ -421,8 +432,8 @@
         self.assertEqual(u'     3 added %s\n' % (fname,), txt)
 
         fname2 = self.info['filename'] + '2'
-        bzr('mv', fname, fname2)
-        bzr('commit', '-m', u'Renamed %s => %s' % (fname, fname2))
+        self.wt.rename_one(fname, fname2)
+        self.wt.commit(u'Renamed %s => %s' % (fname, fname2))
 
         txt = bzr('touching-revisions', fname2)
         expected_txt = (u'     3 added %s\n' 
@@ -464,35 +475,38 @@
         fname2 = self.info['filename'] + '2.txt'
         open(fname2, 'wb').write('ignored\n')
 
-        txt = bzr('unknowns')
-        self.assertEqual(u'"%s"\n' % (fname2,), txt)
+        def check_unknowns(expected):
+            self.assertEqual(expected, list(self.wt.unknowns()))
+
+        check_unknowns([fname2])
 
         bzr('ignore', './' + fname2)
-        txt = bzr('unknowns')
-        self.assertEqual(u'', txt)
+        # After 'ignore' you must re-open the working tree
+        self.wt = self.wt.bzrdir.open_workingtree()
+        check_unknowns([])
 
         fname3 = self.info['filename'] + '3.txt'
         open(fname3, 'wb').write('unknown 3\n')
-        txt = bzr('unknowns')
-        self.assertEqual(u'"%s"\n' % (fname3,), txt)
+        check_unknowns([fname3])
 
         # Ignore should not care what the encoding is
         # (right now it doesn't print anything)
         bzr('ignore', fname3, encoding='ascii')
-        txt = bzr('unknowns')
-        self.assertEqual('', txt)
+        self.wt = self.wt.bzrdir.open_workingtree()
+        check_unknowns([])
 
         # Now try a wildcard match
         fname4 = self.info['filename'] + '4.txt'
         open(fname4, 'wb').write('unknown 4\n')
         bzr('ignore', '*.txt')
-        txt = bzr('unknowns')
-        self.assertEqual('', txt)
+        self.wt = self.wt.bzrdir.open_workingtree()
+        check_unknowns([])
 
+        # and a different wildcard that matches everything
         os.remove('.bzrignore')
         bzr('ignore', self.info['filename'] + '*')
-        txt = bzr('unknowns')
-        self.assertEqual('', txt)
+        self.wt = self.wt.bzrdir.open_workingtree()
+        check_unknowns([])
 
     def test_missing(self):
         bzr = self.run_bzr_decode

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2006-07-03 13:48:52 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-07-04 15:04:11 +0000
@@ -57,36 +57,6 @@
 
 class TestCommands(ExternalBase):
 
-    def test_whoami(self):
-        # this should always identify something, if only "john@localhost"
-        self.runbzr("whoami")
-        self.runbzr("whoami --email")
-
-        self.assertEquals(self.runbzr("whoami --email",
-                                      backtick=True).count('@'), 1)
-        
-    def test_whoami_branch(self):
-        """branch specific user identity works."""
-        self.runbzr('init')
-        b = bzrlib.branch.Branch.open('.')
-        b.control_files.put_utf8('email', 'Branch Identity <branch@identi.ty>')
-        bzr_email = os.environ.get('BZREMAIL')
-        if bzr_email is not None:
-            del os.environ['BZREMAIL']
-        whoami = self.runbzr("whoami",backtick=True)
-        whoami_email = self.runbzr("whoami --email",backtick=True)
-        self.assertTrue(whoami.startswith('Branch Identity <branch@identi.ty>'))
-        self.assertTrue(whoami_email.startswith('branch@identi.ty'))
-        # Verify that the environment variable overrides the value 
-        # in the file
-        os.environ['BZREMAIL'] = 'Different ID <other@environ.ment>'
-        whoami = self.runbzr("whoami",backtick=True)
-        whoami_email = self.runbzr("whoami --email",backtick=True)
-        self.assertTrue(whoami.startswith('Different ID <other@environ.ment>'))
-        self.assertTrue(whoami_email.startswith('other@environ.ment'))
-        if bzr_email is not None:
-            os.environ['BZREMAIL'] = bzr_email
-
     def test_nick_command(self):
         """bzr nick for viewing, setting nicknames"""
         os.mkdir('me.dev')

=== added file 'bzrlib/tests/blackbox/test_whoami.py'
--- bzrlib/tests/blackbox/test_whoami.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_whoami.py	2006-07-03 08:52:44 +0000
@@ -0,0 +1,95 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr whoami."""
+
+import os
+
+import bzrlib
+from bzrlib.branch import Branch
+from bzrlib.tests.blackbox import ExternalBase
+
+
+class TestWhoami(ExternalBase):
+
+    def test_whoami(self):
+        # this should always identify something, if only "john@localhost"
+        out = self.run_bzr("whoami")[0]
+        self.assertTrue(len(out) > 0)
+        self.assertEquals(1, out.count('@'))
+
+        out = self.run_bzr("whoami", "--email")[0]
+        self.assertTrue(len(out) > 0)
+        self.assertEquals(1, out.count('@'))
+        
+    def test_whoami_branch(self):
+        """branch specific user identity works."""
+        wt = self.make_branch_and_tree('.')
+        b = bzrlib.branch.Branch.open('.')
+        b.get_config().set_user_option('email', 'Branch Identity <branch@identi.ty>')
+        bzr_email = os.environ.get('BZREMAIL')
+        if bzr_email is not None:
+            del os.environ['BZREMAIL']
+        try:
+            whoami = self.run_bzr("whoami")[0]
+            self.assertEquals('Branch Identity <branch@identi.ty>\n', whoami)
+            whoami_email = self.run_bzr("whoami", "--email")[0]
+            self.assertEquals('branch@identi.ty\n', whoami_email)
+
+            # Verify that the environment variable overrides the value 
+            # in the file
+            os.environ['BZREMAIL'] = 'Different ID <other@environ.ment>'
+            whoami = self.run_bzr("whoami")[0]
+            self.assertEquals('Different ID <other@environ.ment>\n', whoami)
+            whoami_email = self.run_bzr("whoami", "--email")[0]
+            self.assertEquals('other@environ.ment\n', whoami_email)
+        finally:
+            if bzr_email is not None:
+                os.environ['BZREMAIL'] = bzr_email
+
+    def test_whoami_utf8(self):
+        """verify that an identity can be in utf-8."""
+        wt = self.make_branch_and_tree('.')
+        self.run_bzr('whoami', u'Branch Identity \u20ac <branch@identi.ty>', encoding='utf-8')
+        bzr_email = os.environ.get('BZREMAIL')
+        if bzr_email is not None:
+            del os.environ['BZREMAIL']
+        try:
+            whoami = self.run_bzr("whoami", encoding='utf-8')[0]
+            self.assertEquals('Branch Identity \xe2\x82\xac <branch@identi.ty>\n', whoami)
+            whoami_email = self.run_bzr("whoami", "--email", encoding='utf-8')[0]
+            self.assertEquals('branch@identi.ty\n', whoami_email)
+        finally:
+            if bzr_email is not None:
+                os.environ['BZREMAIL'] = bzr_email
+
+    def test_whoami_ascii(self):
+        """verify that whoami doesn't totally break when in utf-8, using an ascii encoding."""
+        wt = self.make_branch_and_tree('.')
+        b = bzrlib.branch.Branch.open('.')
+        b.get_config().set_user_option('email', u'Branch Identity \u20ac <branch@identi.ty>')
+        bzr_email = os.environ.get('BZREMAIL')
+        if bzr_email is not None:
+            del os.environ['BZREMAIL']
+        try:
+            whoami = self.run_bzr("whoami", encoding='ascii')[0]
+            self.assertEquals('Branch Identity ? <branch@identi.ty>\n', whoami)
+            whoami_email = self.run_bzr("whoami", "--email", encoding='ascii')[0]
+            self.assertEquals('branch@identi.ty\n', whoami_email)
+        finally:
+            if bzr_email is not None:
+                os.environ['BZREMAIL'] = bzr_email

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2006-06-22 15:42:26 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2006-07-04 17:23:00 +0000
@@ -29,6 +29,7 @@
                            NotBranchError,
                            )
 from bzrlib.osutils import getcwd
+import bzrlib.revision
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
 from bzrlib.trace import mutter
@@ -358,6 +359,12 @@
         tree.branch.generate_revision_history(rev1)
         self.assertEqual(orig_history, tree.branch.revision_history())
 
+    def test_generate_revision_history_NULL_REVISION(self):
+        tree = self.make_branch_and_tree('.')
+        rev1 = tree.commit('foo')
+        tree.branch.generate_revision_history(bzrlib.revision.NULL_REVISION)
+        self.assertEqual([], tree.branch.revision_history())
+
 
 class ChrootedTests(TestCaseWithBranch):
     """A support class that provides readonly urls outside the local namespace.

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2006-06-25 21:04:37 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2006-07-04 17:23:00 +0000
@@ -388,6 +388,9 @@
                           'rev3':['rev2'],
                           },
                          self.bzrdir.open_repository().get_revision_graph('rev3'))
+        # and we can ask for the NULLREVISION graph
+        self.assertEqual({},
+            self.bzrdir.open_repository().get_revision_graph(NULL_REVISION))
 
     def test_get_revision_graph_with_ghosts(self):
         # we can get a graph object with roots, ghosts, ancestors and
@@ -410,6 +413,10 @@
                           'rev4':{},
                           },
                           graph.get_descendants())
+        # and we can ask for the NULLREVISION graph
+        graph = repo.get_revision_graph_with_ghosts([NULL_REVISION])
+        self.assertEqual({}, graph.get_ancestors())
+        self.assertEqual({}, graph.get_descendants())
 
 
 class TestCaseWithCorruptRepository(TestCaseWithRepository):

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2006-06-20 05:32:16 +0000
+++ bzrlib/tests/test_diff.py	2006-07-06 13:51:22 +0000
@@ -205,10 +205,15 @@
         # set the date stamps for files in the working tree to known values
         os.utime('file1', (1144195200, 1144195200)) # 2006-04-05 00:00:00 UTC
 
-    def get_diff(self, tree1, tree2):
+    def get_diff(self, tree1, tree2, specific_files=None, working_tree=None):
         output = StringIO()
-        show_diff_trees(tree1, tree2, output,
-                        old_label='old/', new_label='new/')
+        if working_tree is not None:
+            extra_trees = (working_tree,)
+        else:
+            extra_trees = ()
+        show_diff_trees(tree1, tree2, output, specific_files=specific_files,
+                        extra_trees=extra_trees, old_label='old/', 
+                        new_label='new/')
         return output.getvalue()
 
     def test_diff_rev_tree_working_tree(self):
@@ -275,6 +280,29 @@
 
 ''')
 
+    def test_show_diff_specified(self):
+        """A working tree filename can be used to identify a file"""
+        self.wt.rename_one('file1', 'file1b')
+        old_tree = self.b.repository.revision_tree('rev-1')
+        new_tree = self.b.repository.revision_tree('rev-4')
+        out = self.get_diff(old_tree, new_tree, specific_files=['file1b'], 
+                            working_tree=self.wt)
+        self.assertContainsRe(out, 'file1\t')
+
+    def test_recursive_diff(self):
+        """Children of directories are matched"""
+        os.mkdir('dir1')
+        os.mkdir('dir2')
+        self.wt.add(['dir1', 'dir2'])
+        self.wt.rename_one('file1', 'dir1/file1')
+        old_tree = self.b.repository.revision_tree('rev-1')
+        new_tree = self.b.repository.revision_tree('rev-4')
+        out = self.get_diff(old_tree, new_tree, specific_files=['dir1'], 
+                            working_tree=self.wt)
+        self.assertContainsRe(out, 'file1\t')
+        out = self.get_diff(old_tree, new_tree, specific_files=['dir2'], 
+                            working_tree=self.wt)
+        self.assertNotContainsRe(out, 'file1\t')
 
 class TestPatienceDiffLib(TestCase):
 

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2006-06-20 03:57:11 +0000
+++ bzrlib/tests/test_revision.py	2006-07-04 17:23:00 +0000
@@ -23,7 +23,8 @@
 from bzrlib.graph import Graph
 from bzrlib.revision import (find_present_ancestors, combined_graph,
                              common_ancestor,
-                             is_ancestor, MultipleRevisionSources)
+                             is_ancestor, MultipleRevisionSources,
+                             NULL_REVISION)
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.trace import mutter
 from bzrlib.workingtree import WorkingTree
@@ -221,6 +222,12 @@
         self.assertTrue(common_ancestor(revisions_2[6], revisions[5], sources),
                         (revisions[4], revisions_2[5]))
         self.assertEqual(None, common_ancestor(None, revisions[5], sources))
+        self.assertEqual(NULL_REVISION,
+            common_ancestor(NULL_REVISION, NULL_REVISION, sources))
+        self.assertEqual(NULL_REVISION,
+            common_ancestor(revisions[0], NULL_REVISION, sources))
+        self.assertEqual(NULL_REVISION,
+            common_ancestor(NULL_REVISION, revisions[0], sources))
 
     def test_combined(self):
         """combined_graph

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-06-29 16:51:04 +0000
+++ bzrlib/tests/test_selftest.py	2006-07-03 05:38:39 +0000
@@ -507,14 +507,15 @@
         #           1        0            ???         ???       ???(sleep) 
         # and then repeated but with 'world', rather than 'hello'.
         # this should appear in the output stream of our test result.
-        self.assertContainsRe(result_stream.getvalue(), 
-            r"LSProf output for <type 'unicode'>\(\('hello',\), {'errors': 'replace'}\)\n"
-            r" *CallCount *Recursive *Total\(ms\) *Inline\(ms\) *module:lineno\(function\)\n"
-            r"( +1 +0 +0\.\d+ +0\.\d+ +<method 'disable' of '_lsprof\.Profiler' objects>\n)?"
-            r"LSProf output for <type 'unicode'>\(\('world',\), {'errors': 'replace'}\)\n"
-            r" *CallCount *Recursive *Total\(ms\) *Inline\(ms\) *module:lineno\(function\)\n"
-            r"( +1 +0 +0\.\d+ +0\.\d+ +<method 'disable' of '_lsprof\.Profiler' objects>\n)?"
-            )
+        output = result_stream.getvalue()
+        self.assertContainsRe(output,
+            r"LSProf output for <type 'unicode'>\(\('hello',\), {'errors': 'replace'}\)")
+        self.assertContainsRe(output,
+            r" *CallCount *Recursive *Total\(ms\) *Inline\(ms\) *module:lineno\(function\)\n")
+        self.assertContainsRe(output,
+            r"( +1 +0 +0\.\d+ +0\.\d+ +<method 'disable' of '_lsprof\.Profiler' objects>\n)?")
+        self.assertContainsRe(output,
+            r"LSProf output for <type 'unicode'>\(\('world',\), {'errors': 'replace'}\)\n")
 
 
 class TestRunner(TestCase):

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2006-06-07 11:13:02 +0000
+++ bzrlib/tests/test_transform.py	2006-07-05 04:22:00 +0000
@@ -20,8 +20,9 @@
 from bzrlib.conflicts import (DuplicateEntry, DuplicateID, MissingParent,
                               UnversionedParent, ParentLoop)
 from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
-                           ReusingTransform, CantMoveRoot, NotVersionedError,
-                           ExistingLimbo, ImmortalLimbo, LockError)
+                           ReusingTransform, CantMoveRoot, 
+                           PathsNotVersionedError, ExistingLimbo,
+                           ImmortalLimbo, LockError)
 from bzrlib.osutils import file_kind, has_symlinks, pathjoin
 from bzrlib.merge import Merge3Merger
 from bzrlib.tests import TestCaseInTempDir, TestSkipped, TestCase
@@ -492,7 +493,7 @@
         create.apply()
         self.assertEqual(find_interesting(wt, wt, ['vfile']),
                          set(['myfile-id']))
-        self.assertRaises(NotVersionedError, find_interesting, wt, wt,
+        self.assertRaises(PathsNotVersionedError, find_interesting, wt, wt,
                           ['uvfile'])
 
     def test_set_executability_order(self):

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-07-03 13:48:52 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-07-05 04:22:00 +0000
@@ -22,7 +22,7 @@
 import bzrlib
 from bzrlib import branch, bzrdir, errors, urlutils, workingtree
 from bzrlib.errors import (NotBranchError, NotVersionedError, 
-                           UnsupportedOperation)
+                           UnsupportedOperation, PathsNotVersionedError)
 from bzrlib.osutils import pathjoin, getcwd, has_symlinks
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
@@ -128,7 +128,7 @@
         self.build_tree(['hello.txt'])
         file('hello.txt', 'w').write('initial hello')
 
-        self.assertRaises(NotVersionedError,
+        self.assertRaises(PathsNotVersionedError,
                           tree.revert, ['hello.txt'])
         tree.add(['hello.txt'])
         tree.commit('create initial hello.txt')

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2006-07-03 13:48:52 +0000
+++ bzrlib/transform.py	2006-07-06 13:51:22 +0000
@@ -26,6 +26,7 @@
                             delete_any)
 from bzrlib.progress import DummyProgress, ProgressPhase
 from bzrlib.trace import mutter, warning
+from bzrlib import tree
 import bzrlib.ui 
 import bzrlib.urlutils as urlutils
 
@@ -999,36 +1000,7 @@
 def find_interesting(working_tree, target_tree, filenames):
     """Find the ids corresponding to specified filenames."""
     trees = (working_tree, target_tree)
-    if not filenames:
-        interesting_ids = None
-    else:
-        interesting_ids = set()
-        for tree_path in filenames:
-            not_found = True
-            for tree in trees:
-                file_id = tree.inventory.path2id(tree_path)
-                if file_id is not None:
-                    interesting_ids.add(file_id)
-                    not_found = False
-            if not_found:
-                raise NotVersionedError(path=tree_path)
-        
-        pending = interesting_ids
-        # now handle children of interesting ids
-        # we loop so that we handle all children of each id in both trees
-        while len(pending) > 0:
-            new_pending = set()
-            for file_id in pending:
-                for tree in trees:
-                    if file_id not in tree:
-                        continue
-                    entry = tree.inventory[file_id]
-                    for child in getattr(entry, 'children', {}).itervalues():
-                        if child.file_id not in interesting_ids:
-                            new_pending.add(child.file_id)
-            interesting_ids.update(new_pending)
-            pending = new_pending
-    return interesting_ids
+    return tree.find_ids_across_trees(filenames, trees)
 
 
 def change_entry(tt, file_id, working_tree, target_tree, 

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-06-18 03:39:50 +0000
+++ bzrlib/tree.py	2006-07-06 13:51:22 +0000
@@ -22,6 +22,7 @@
 
 import bzrlib
 from bzrlib.errors import BzrError, BzrCheckError
+from bzrlib import errors
 from bzrlib.inventory import Inventory
 from bzrlib.osutils import fingerprint_file
 import bzrlib.revision
@@ -335,4 +336,74 @@
             yield (old_name, new_name)
             
 
-
+def find_ids_across_trees(filenames, trees, require_versioned=True):
+    """Find the ids corresponding to specified filenames.
+    
+    All matches in all trees will be used, and all children of matched
+    directories will be used.
+
+    :param filenames: The filenames to find file_ids for
+    :param trees: The trees to find file_ids within
+    :param require_versioned: if true, all specified filenames must occur in
+    at least one tree.
+    :return: a set of file ids for the specified filenames and their children.
+    """
+    if not filenames:
+        return None
+    specified_ids = _find_filename_ids_across_trees(filenames, trees, 
+                                                    require_versioned)
+    return _find_children_across_trees(specified_ids, trees)
+
+
+def _find_filename_ids_across_trees(filenames, trees, require_versioned):
+    """Find the ids corresponding to specified filenames.
+    
+    All matches in all trees will be used.
+
+    :param filenames: The filenames to find file_ids for
+    :param trees: The trees to find file_ids within
+    :param require_versioned: if true, all specified filenames must occur in
+    at least one tree.
+    :return: a set of file ids for the specified filenames
+    """
+    not_versioned = []
+    interesting_ids = set()
+    for tree_path in filenames:
+        not_found = True
+        for tree in trees:
+            file_id = tree.inventory.path2id(tree_path)
+            if file_id is not None:
+                interesting_ids.add(file_id)
+                not_found = False
+        if not_found:
+            not_versioned.append(tree_path)
+    if len(not_versioned) > 0 and require_versioned:
+        raise errors.PathsNotVersionedError(not_versioned)
+    return interesting_ids
+
+
+def _find_children_across_trees(specified_ids, trees):
+    """Return a set including specified ids and their children
+    
+    All matches in all trees will be used.
+
+    :param trees: The trees to find file_ids within
+    :return: a set containing all specified ids and their children 
+    """
+    interesting_ids = set(specified_ids)
+    pending = interesting_ids
+    # now handle children of interesting ids
+    # we loop so that we handle all children of each id in both trees
+    while len(pending) > 0:
+        new_pending = set()
+        for file_id in pending:
+            for tree in trees:
+                if file_id not in tree:
+                    continue
+                entry = tree.inventory[file_id]
+                for child in getattr(entry, 'children', {}).itervalues():
+                    if child.file_id not in interesting_ids:
+                        new_pending.add(child.file_id)
+        interesting_ids.update(new_pending)
+        pending = new_pending
+    return interesting_ids

=== modified file 'doc/README.1st'
--- doc/README.1st	2006-03-14 16:57:35 +0000
+++ doc/README.1st	2006-06-29 23:25:48 +0000
@@ -1,6 +1,6 @@
 These documents originally came from the Bazaar-NG wiki hosted at
-http://bazaar-vcs.org. These documents, though maintained, These documenets
-can fall behind when Bazaar-NG is under heavy development.
+http://bazaar-vcs.org. These documents, though maintained, can fall behind
+when Bazaar-NG is under heavy development.
 
 The documents on the wiki take precedence in the event of a discrepancy
 between what is documented here and what is documented on the wiki.

=== modified file 'doc/setting_up_email.txt'
--- doc/setting_up_email.txt	2006-06-16 15:59:24 +0000
+++ doc/setting_up_email.txt	2006-06-29 23:26:31 +0000
@@ -31,14 +31,25 @@
 
     % bzr whoami
     Joe Cool <joe@example.com>
+    
+Setting email via the 'whoami' command
+======================================
+You can use the whoami command to set your email globally::
+
+    % bzr whoami 'Joe Cool <joe@example.com>'
+
+or only for the current branch::
+
+    % bzr whoami --branch 'Joe Cool <joe@example.com>'
+
+These modify your global bazaar.conf or branch branch.conf file, respectively.
 
 Setting email via default ini file
 ==================================
-The first method is using the default ini file.  To use the default ini
-method, create the file `$HOME/.bazaar/bazaar.conf` (on Windows this is
-`%APPDATA%\bazaar\2.0\bazaar.conf`) and set an email address as shown
-below.  Please note that the word DEFAULT is case sensitive, and must be in
-upper-case.::
+To use the default ini file, create the file `$HOME/.bazaar/bazaar.conf` (on
+Windows this is `%APPDATA%\bazaar\2.0\bazaar.conf`) and set an email address
+as shown below.  Please note that the word DEFAULT is case sensitive, and
+must be in upper-case.::
 
     [DEFAULT]
     email=Your Name <name@isp.com>

=== modified file 'doc/tutorial.txt'
--- doc/tutorial.txt	2006-06-17 19:01:15 +0000
+++ doc/tutorial.txt	2006-06-29 23:27:05 +0000
@@ -117,6 +117,16 @@
 address by looking up your username and hostname. If you don't like the
 guess that Bazaar-NG makes, then three options exist:
 
+ 1. Set an email address via ``bzr whoami``.  This is the simplest way.
+ To set a global identity, use::
+ 
+     % bzr whoami 'Your Name <email@example.com>'
+ 
+ If you'd like to use a different address for a specific branch, enter
+ the branch folder and use::
+ 
+     % bzr whoami --branch 'Your Name <email@example.com>'
+ 
  1. Setting the email address in the
  ``~/.bazaar/bazaar.conf`` by adding the following lines. Please note that
  ``[DEFAULT]`` is case sensitive::
@@ -124,9 +134,9 @@
     [DEFAULT]
     email= Your Name <email@isp.com>
  
- 1. Override the previous setting on a
- branch by branch basis by creating a branch section in
- ``~/.bazaar/locations.conf`` by adding the following lines::
+ As above, you can override this settings on a branch by branch basis by
+ creating a branch section in ``~/.bazaar/locations.conf`` and adding the
+ following lines::
 
     [/the/directory/to/the/branch]
     email=Your Name <email@isp.com>

